==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/score.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: score.c,v 1.12 2003/10/17 18:42:13 lwa Exp $";

#include "config.h"
#include <stdlib.h>
#include <strings.h>
#include "lmtp.h"

static unsigned int nb_headers(char **hdr, struct headers *hdrlist) {
  unsigned int ret = 0;

  if (**hdr && **hdr != ' ' && **hdr != '\t' &&
      **hdr != '<' && **hdr != '>' && **hdr != '=') {
    int stop = 0;
    while(!stop) {
      char *xhdr = *hdr;
      char *chdr = *hdr;
      while(1) {
	if (*xhdr == 0 || *xhdr == ' '   || *xhdr == '\t' ||
	    *xhdr == '<' || *xhdr == '>' || *xhdr == '=') {
	  stop = 1;
	  break;
	}
	if (*xhdr == ':')
	  break;
	xhdr++;
      }
      if (xhdr>chdr) {
	struct headers *hl = hdrlist;
	while(hl) {
	  if (hl->dd_len == xhdr-chdr &&
	      strncasecmp(hl->header, chdr, xhdr-chdr) == 0)
	    ret ++;
	  hl = hl ->next;
	}
      }
      while (*xhdr == ':')
	xhdr++;
      *hdr = xhdr;
    }
  } else {
    struct headers *hl = hdrlist;
    while(hl) {
      ret ++;
      hl = hl ->next;
    }
  }
  return ret;
}


score_t readscore(struct fstate *state, char **buf) {
  score_t s=0;
  char *rest;
  s=A2S(*buf, &rest, 10);
  
  if (rest==*buf) {
    s=1;
    if (*rest=='+') 
      rest++;
    else if (*rest=='-') {
      rest++;
      s=-1;
    }
  }
  switch(*rest) {
  case 'g': case 'G': s*=1024 * 1024 * 1024 ; rest++; break;
  case 'm': case 'M': s*=1024 * 1024; rest++; break;
  case 'k': case 'K': s*=1024; rest++; break;
  case 's': case 'S': s*=state->msgsize; rest++; break;
  case 'l': case 'L': s*=state->nblines; rest++; break;
  case 'r': case 'R': s*=state->nrcpts_r; rest++; break;
  case 'i': case 'I': s=0; rest++; break; /* disabled */
  case 't': case 'T': s*=scoretime(state, &rest); break;
  case 'v': case 'V': s*=av_scan(state); rest++; break;
  case 'a': case 'A':
    rest++;
    spam_assassin(state, NULL);
    s*=state->spamassassin_score;
    break;
  case '?': s*=(double)random()/0x80000000; rest++; break;
  case 'h': rest++; s*=nb_headers(&rest, state->hdrlist); break;
  default:
    break;
  }
  if (*buf == rest)
    s=state->score; /* empty returns current score */
  else
    *buf=rest;
  return s;
}

#define SOP_ERR -1
#define SOP_NONE 0
#define SOP_EQ 1
#define SOP_GE 2
#define SOP_LE 3
#define SOP_GT 4
#define SOP_LT 5
#define SOP_NE 6

int test_scalar(struct fstate *state, char **buf) {
  int op = SOP_NONE;
  score_t acc;
  char *rest = *buf;
  acc = readscore(state, &rest);
  switch(*rest) {
  case '<':
    if (*++rest == '=') {
      op = SOP_LE; ++rest;
    } else if (*rest == '>') {
      op = SOP_NE; ++rest;
    } else {
      op = SOP_LT;
    }
    break;
  case '=': op = SOP_EQ; rest ++; break;
  case '>': 
    if (*++rest == '=') {
      op = SOP_GE; ++rest;
    } else {
      op = SOP_GT;
    }
    break; 
  case '!':
    if (*++rest == '=') {
      op = SOP_NE; ++rest ;
    } else {
      --rest;
    }
    break;
  default:
    break;
  }
  if (op != SOP_NONE) {
    score_t acc2 = readscore(state, &rest);
    switch(op) {
    case SOP_LE: acc = (acc <= acc2); break;
    case SOP_LT: acc = (acc < acc2); break;
    case SOP_GE: acc = (acc >= acc2); break;
    case SOP_GT: acc = (acc > acc2); break;
    case SOP_NE: acc = (acc != acc2); break;
    case SOP_EQ: acc = (acc == acc2); break;
    }
  }
  *buf = rest;
  return acc != 0;
}

==> lmtpd-0.10.4.tar.gz: lmtpd-0.10.4/score.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: score.c,v 1.13 2004/10/26 09:59:20 lwa Exp $";


#include <stdlib.h>
#include <strings.h>
#include "lmtp.h"

static unsigned int nb_headers(char **hdr, struct headers *hdrlist) {
  unsigned int ret = 0;

  if (**hdr && **hdr != ' ' && **hdr != '\t' &&
      **hdr != '<' && **hdr != '>' && **hdr != '=') {
    int stop = 0;
    while(!stop) {
      char *xhdr = *hdr;
      char *chdr = *hdr;
      while(1) {
	if (*xhdr == 0 || *xhdr == ' '   || *xhdr == '\t' ||
	    *xhdr == '<' || *xhdr == '>' || *xhdr == '=') {
	  stop = 1;
	  break;
	}
	if (*xhdr == ':')
	  break;
	xhdr++;
      }
      if (xhdr>chdr) {
	struct headers *hl = hdrlist;
	while(hl) {
	  if (hl->dd_len == xhdr-chdr &&
	      strncasecmp(hl->header, chdr, xhdr-chdr) == 0)
	    ret ++;
	  hl = hl ->next;
	}
      }
      while (*xhdr == ':')
	xhdr++;
      *hdr = xhdr;
    }
  } else {
    struct headers *hl = hdrlist;
    while(hl) {
      ret ++;
      hl = hl ->next;
    }
  }
  return ret;
}


score_t readscore(struct fstate *state, char **buf) {
  score_t s=0;
  char *rest;
  s=A2S(*buf, &rest, 10);
  
  if (rest==*buf) {
    s=1;
    if (*rest=='+') 
      rest++;
    else if (*rest=='-') {
      rest++;
      s=-1;
    }
  }
  switch(*rest) {
  case 'g': case 'G': s*=1024 * 1024 * 1024 ; rest++; break;
  case 'm': case 'M': s*=1024 * 1024; rest++; break;
  case 'k': case 'K': s*=1024; rest++; break;
  case 's': case 'S': s*=state->msgsize; rest++; break;
  case 'l': case 'L': s*=state->nblines; rest++; break;
  case 'r': case 'R': s*=state->nrcpts_r; rest++; break;
  case 'i': case 'I': s=0; rest++; break; /* disabled */
  case 't': case 'T': s*=scoretime(state, &rest); break;
  case 'v': case 'V': s*=av_scan(state); rest++; break;
  case 'a': case 'A':
    rest++;
    spam_assassin(state, NULL);
    s*=state->spamassassin_score;
    break;
  case '?': s*=(double)random()/0x80000000; rest++; break;
  case 'h': rest++; s*=nb_headers(&rest, state->hdrlist); break;
  default:
    break;
  }
  if (*buf == rest)
    s=state->score; /* empty returns current score */
  else
    *buf=rest;
  return s;
}

#define SOP_ERR -1
#define SOP_NONE 0
#define SOP_EQ 1
#define SOP_GE 2
#define SOP_LE 3
#define SOP_GT 4
#define SOP_LT 5
#define SOP_NE 6

int test_scalar(struct fstate *state, char **buf) {
  int op = SOP_NONE;
  score_t acc;
  char *rest = *buf;
  acc = readscore(state, &rest);
  switch(*rest) {
  case '<':
    if (*++rest == '=') {
      op = SOP_LE; ++rest;
    } else if (*rest == '>') {
      op = SOP_NE; ++rest;
    } else {
      op = SOP_LT;
    }
    break;
  case '=': op = SOP_EQ; rest ++; break;
  case '>': 
    if (*++rest == '=') {
      op = SOP_GE; ++rest;
    } else {
      op = SOP_GT;
    }
    break; 
  case '!':
    if (*++rest == '=') {
      op = SOP_NE; ++rest ;
    } else {
      --rest;
    }
    break;
  default:
    break;
  }
  if (op != SOP_NONE) {
    score_t acc2 = readscore(state, &rest);
    switch(op) {
    case SOP_LE: acc = (acc <= acc2); break;
    case SOP_LT: acc = (acc < acc2); break;
    case SOP_GE: acc = (acc >= acc2); break;
    case SOP_GT: acc = (acc > acc2); break;
    case SOP_NE: acc = (acc != acc2); break;
    case SOP_EQ: acc = (acc == acc2); break;
    }
  }
  *buf = rest;
  return acc != 0;
}
